#extension GL_OES_EGL_image_external : require
precision mediump float;
varying vec4 vTextureCoord;
uniform samplerExternalOES sTexture;

uniform highp float S;
uniform highp float H;
uniform highp float L;

highp vec3 rgb2hsv(highp vec3 c){
    highp vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    highp vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    highp vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    highp float d = q.x - min(q.w, q.y);
    highp float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

highp vec3 hsv2rgb(highp vec3 c){
    highp vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    highp vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {
    highp vec4 rgba = texture2D(sTexture, vTextureCoord.xy / vTextureCoord.z);
    highp vec3 hsl = rgb2hsv(rgba.xyz);
    if (S != 1.0) hsl.y = hsl.y * S;
    if (H != 0.0) hsl.x = H;
    if (hsl.x < 0.0) hsl.x = hsl.x + 1.0;
    else if (hsl.x > 1.0) hsl.x = hsl.x-1.0;
    if (L != 1.0) hsl.z = hsl.z * L;
    highp vec3 rgb = hsv2rgb(hsl);
    gl_FragColor = vec4(rgb, rgba.w);
}